#include "stdafx.h"
#include <iostream>
#include <iomanip>
#include <cstdlib> // exit function prototype
#include "arrayClass.h" // arrayClass class definition
using namespace std;

// default constructor for class arrayClass (default size 10)
arrayClass::arrayClass( int arrayClassSize )
{
   size = ( arrayClassSize > 0 ? arrayClassSize : 10 ); // validate arrayClassSize
   ptr = new int[ size ]; // create space for pointer-based arrayClass

   for ( int i = 0; i < size; i++ )
      ptr[ i ] = 0; // set pointer-based arrayClass element
} // end arrayClass default constructor

// copy constructor for class arrayClass;
// must receive a reference to prevent infinite recursion
arrayClass::arrayClass( const arrayClass &arrayClassToCopy ) 
   : size( arrayClassToCopy.size )
{
   ptr = new int[ size ]; // create space for pointer-based arrayClass

   for ( int i = 0; i < size; i++ )
      ptr[ i ] = arrayClassToCopy.ptr[ i ]; // copy into object
} // end arrayClass copy constructor

// destructor for class arrayClass
arrayClass::~arrayClass()
{
   delete [] ptr; // release pointer-based arrayClass space
} // end destructor

// return number of elements of arrayClass
int arrayClass::getSize() const
{
   return size; // number of elements in arrayClass
} // end function getSize

// overloaded assignment operator;
// const return avoids: ( a1 = a2 ) = a3
const arrayClass &arrayClass::operator=( const arrayClass &right )
{
   if ( &right != this ) // avoid self-assignment
   {
      // for arrayClasss of different sizes, deallocate original
      // left-side arrayClass, then allocate new left-side arrayClass
      if ( size != right.size )
      {
         delete [] ptr; // release space
         size = right.size; // resize this object
         ptr = new int[ size ]; // create space for arrayClass copy
      } // end inner if

      for ( int i = 0; i < size; i++ )
         ptr[ i ] = right.ptr[ i ]; // copy arrayClass into object
   } // end outer if

   return *this; // enables x = y = z, for example
} // end function operator=

// determine if two arrayClasss are equal and
// return true, otherwise return false
bool arrayClass::operator==( const arrayClass &right ) const
{
   if ( size != right.size )
      return false; // arrayClasss of different number of elements

   for ( int i = 0; i < size; i++ )
      if ( ptr[ i ] != right.ptr[ i ] )
         return false; // arrayClass contents are not equal

   return true; // arrayClasss are equal
} // end function operator==

// overloaded subscript operator for non-const arrayClasss;
// reference return creates a modifiable lvalue
int &arrayClass::operator[]( int subscript )
{
   // check for subscript out-of-range error
   if ( subscript < 0 || subscript >= size )
   {
      cerr << "\nError: Subscript " << subscript 
         << " out of range" << endl;
	  cin.get();
      exit( 1 ); // terminate program; subscript out of range
   } // end if

   return ptr[ subscript ]; // reference return
} // end function operator[]

// overloaded subscript operator for const arrayClasss
// const reference return creates an rvalue
int arrayClass::operator[]( int subscript ) const
{
   // check for subscript out-of-range error
   if ( subscript < 0 || subscript >= size )
   {
      cerr << "\nError: Subscript " << subscript 
         << " out of range" << endl;
      exit( 1 ); // terminate program; subscript out of range
   } // end if

   return ptr[ subscript ]; // returns copy of this element
} // end function operator[]

// overloaded input operator for class arrayClass;
// inputs values for entire arrayClass
istream &operator>>( istream &input, arrayClass &a )
{
   for ( int i = 0; i < a.size; i++ )
      input >> a.ptr[ i ];
   return input; // enables cin >> x >> y;
} // end function 

// overloaded output operator for class arrayClass 
ostream &operator<<( ostream &output, const arrayClass &a )
{
   int i;

   // output private ptr-based arrayClass 
   for ( i = 0; i < a.size; i++ )
   {
      output << setw( 12 ) << a.ptr[ i ];

      if ( ( i + 1 ) % 4 == 0 ) // 4 numbers per row of output
         output << endl;
   } // end for

   if ( i % 4 != 0 ) // end last line of output
      output << endl;
   return output; // enables cout << x << y;
} // end function operator<<
